%
%                            Copyright 2011 Zilmar de Souza Junior,
%						                    Marcos Moecke 
%
%
%
%    Atualiza_Interface_fm is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    Atualiza_Interface_fm is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with Atualiza_Interface_fm. If not, see <http://www.gnu.org/licenses/>.
%

function [mod,s] = Atualiza_Interface_fm(handles)  
    %% Valores recebidos da interface
    %Frequencia da portadora
    fp = handles.metricdata.freqp;
    %Frequencia da modulante
    fm = handles.freqm;
    % V de pico da portadora
    vpp = handles.metricdata.vpico;
    % Indice de modula��o
    beta=handles.metricdata.m;
    % V de pico da modulante
    vmp = handles.vmodulante;
    
    % beta para sinc
    bsinc = handles.bsinc;
    
    
    %% Calculos para determinar a portadora, modulante e o sinal modulado
    % Intervalo = 1:
    dur=1;
    % Frequencia de amostragem:
    fs=1e5;
    % Periodo de amostragem:
    Ts = 1/fs;
    
    % Modulante    
    modulante=handles.modulante; %Recebe a funco do menu;
    if(modulante ~= 6)
        % Vetor tempo:
        t=-dur : Ts : dur - Ts;
        % Cria��o Vetor Frequ�ncia 
        df = (1 / (2*dur)) / 16;
        f = -fs/2 : df : fs/2-df;
        mod=DefineMod(modulante,vmp,fm,t,bsinc);
    else
        [mod,dur,t,df,f]=DefineModAudio(handles.fullPath,fs,Ts,dur);
    end
    
    % Sinal da portadora:
    p=vpp*cos(2*pi*fp.*t);
    
    %Sinal Modulado
    s= vpp.*cos(2*pi*fp*t + beta*Ts*cumsum(mod));
    
    % FFT do sinal modulado no intervalo lf = length(t):
    S=fftshift(fft(s,length(f)))/fs;
    
    % plota a portadora no plot correspondente com a cor vermelha:
    plot(handles.axes_port,t,p,'r');
    
    % define escala e grid
    a = get(handles.axes_port,'YLim');
    set(handles.axes_port,'YLim', [a(1)-(a(2)-a(1))*.1 a(2)+(a(2)-a(1))*.1])
    if(modulante==6)
        set(handles.axes_port,'XLim',[0 dur])%[-2*2*pi/fp 2*2*pi/fp])
    else
        set(handles.axes_port,'XLim',[-1/fp 1/fp])%[-2*2*pi/fp 2*2*pi/fp])
    end
    set(handles.axes_port,'XGrid','on')
    set(handles.axes_port,'YGrid','on')
    
    % plota a modulante no plot correspondente com a cor padrao (azul):
    plot(handles.axes_mod,t,mod);
    % define escala e grid
    a = get(handles.axes_mod,'YLim');
    set(handles.axes_mod,'YLim', [a(1)-(a(2)-a(1))*.1 a(2)+(a(2)-a(1))*.1])
    if(modulante==6)
        set(handles.axes_mod,'XLim', [0 dur])
    else
        set(handles.axes_mod,'XLim', [-3/fm 3/fm])
    end
    set(handles.axes_mod,'XGrid','on')
    set(handles.axes_mod,'YGrid','on')
    
    % plota o sinal modulado no plot correspondente com a cor preta:
    plot(handles.axes_sign,t,s,'k');
    % define escala e grid
    a = get(handles.axes_sign,'YLim');
    set(handles.axes_sign,'YLim', [a(1)-(a(2)-a(1))*.1 a(2)+(a(2)-a(1))*.1])
    if(modulante==6)
        set(handles.axes_sign,'XLim', [0 dur])
    else
        set(handles.axes_sign,'XLim', [-1/fm 1/fm])%[-2*2*pi/fp 2*2*pi/fp])
    end
    set(handles.axes_sign,'XGrid','on')
    set(handles.axes_sign,'YGrid','on')
    
    % valor do eixo omega usado no plot do sinal modulado no dominio da
    % frequencia:
    plot(handles.axes_freq,f,abs(S),'g')
    a = get(handles.axes_freq,'YLim');
    set(handles.axes_freq,'YLim', [a(1)-(a(2)-a(1))*.1 a(2)+(a(2)-a(1))*.1])
    if(modulante==6)
        set(handles.axes_freq,'XLim',[-10*fm-fp fp+10*fm])
    else
        set(handles.axes_freq,'XLim',[-10*fm-fp fp+10*fm])
    end
    
    axes(handles.axes_port)
    xlabel('tempo (s)');
    ylabel('Amplitude (V)');
    axes(handles.axes_mod)
    %title('Sinal modulado no dom�nio do tempo')
    xlabel('tempo (s)');
    ylabel('Amplitude (V)');
    axes(handles.axes_sign)
    xlabel('tempo (s)');
    ylabel('Amplitude (V)');
    axes(handles.axes_freq)
    xlabel('frequ�ncia(Hz)');
    ylabel('Amplitude (V)');
   
end