%
%                            Copyright 2011 Zilmar de Souza Junior,
%						                    Marcos Moecke 
%
%
%
%    Atualiza_Interface_Codigos is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    Atualiza_Interface_Codigos is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with Atualiza_Interface_Codigos. If not, see <http://www.gnu.org/licenses/>.
%

function [sinal,sinalquantizado] = Atualiza_Interface_quantizacao(handles)
    addpath(fullfile('.','lib'));
    
    %maximo valor de quantizacao
    mp=handles.metricdata.mp;
    % numero de niveis
    nNiveis=handles.metricdata.nNiveis;
    % amplitudes
    if handles.metricdata.midTread
        amplitudes = -mp:mp/(nNiveis/2):mp-(2*mp/nNiveis);
    else   
        amplitudes = -mp+(mp/nNiveis):mp/(nNiveis/2):mp-(mp/nNiveis);
    end
    % Amplitudes para nao uniforme
    if handles.metricdata.radiobutton_naoUniforme
        if handles.metricdata.lei
            amplitudes=compand(amplitudes,handles.metricdata.fator,mp,'A/expander');
        else
            amplitudes=compand(amplitudes,handles.metricdata.fator,mp,'mu/expander');
        end
    end
    % Intervalo = 1:
    dur=1;
    % Frequencia de amostragem para plot continuo:
    fs=1e5;    
    % Periodo de amostragem para plot continuo:
    Ts = 1/fs;        
    %Frequencia do sinal
    fsinal = handles.freqsinal;     
    % V de pico do sinal
    vp = handles.vsinal;   
    % beta para sinc
    bsinc = handles.bsinc;    
    % frequencia de amostragem para plot discreto
    fa=handles.metricdata.fa;
    % periodo de amostragem para plot discreto
    Ta = 1/fa;
    
    tiposinal=handles.tiposinal; %Recebe a funcao do menu;
    if(tiposinal ~= 6) % continua caso sinal de entrada seja uma onda pre definida
        % Vetor tempo:
        t= -dur : Ts : dur - Ts;
        tdiscreto= -dur : Ta : dur - Ta;
        % vetor frequ�ncia
        %df = (1 / (2*dur)) / 16;
        df = 1 / (2*dur);
        f = -fs/2 : df : fs/2 - df;
        f2 = -fa/2 : df : fa/2 - df;
        
        sinal=DefineMod(tiposinal,vp,fsinal,t,bsinc);
        sinaldiscreto=DefineMod(tiposinal,vp,fsinal,tdiscreto,bsinc);
    else % sinal caso seja usado um arquivo de som
        [sinal,dur,t,df,f]=DefineModAudio(handles.fullPath,fs,Ts,dur);
        [sinaldiscreto,dur,tdiscreto,df,f2]=DefineModAudio(handles.fullPath,fa,Ta,dur);  
    end

    
    % quantizacao de sinal discreto
    limiares = [-inf, (amplitudes(1:end-1)+amplitudes(2:end))/2, inf];
    [~, ic] = histc(sinaldiscreto, limiares);
    sinalquantizado = amplitudes(ic);

    % plot do sinal original  
    if handles.metricdata.checkbox_sinalOriginal
        plot(handles.axes_sinais,t,sinal);
        set(handles.axes_sinais,'NextPlot','add');
    end
    
    % plot do sinal amostrado
    if handles.metricdata.checkbox_sinalAmostrado
        stem(handles.axes_sinais,tdiscreto,sinaldiscreto,'r');
        set(handles.axes_sinais,'NextPlot','add');
    end
    
    % plot das amostras quantizadas
    if handles.metricdata.checkbox_amostrasQuantizadas
        stem(handles.axes_sinais,tdiscreto,sinalquantizado,'g');
        set(handles.axes_sinais,'NextPlot','add');
    end
    
    % plot do sinal quantizado
    if handles.metricdata.checkbox_sinalQuantizado
        stairs(handles.axes_sinais,tdiscreto,sinalquantizado,'k');
        set(handles.axes_sinais,'NextPlot','add');    
    end
    
    legend('Original','Amostrado','Amostras Quant.','Quantizado');
    set(handles.axes_sinais,'NextPlot','replace'); %equivalente ao hold off
            
    % define os limites do eixo X no tempo para ondas predefinidas ou arquivo de som
    if tiposinal == 6
        set(handles.axes_sinais,'xlim',[0 dur]);
    else
        set(handles.axes_sinais,'xlim',[0 3/fsinal]);
    end
    
        
     
    

    % transformada de fourier
    SINAL = fftshift(fft(sinal, length(f)))/fs;
    
    MM = zeros(size(f));
    MM(fs/2 + 1 : fa/df : end - fa/df) = 1;    
    MM(fs/2 + 1 - fa/df: -fa/df : 1 + fa/df) = 1;
    SINALAMOSTRADO = fftconv(SINAL, MM);
    
    %ell = round(fa / df);
    %SINALAMOSTRADO = zeros(size(f));
    %for i = 1 : floor(fs/(2*fa))
    %    TMP = circshift(SINAL', i*ell)';
    %    TMP(1:i*ell) = 0;
    %    SINALAMOSTRADO = SINALAMOSTRADO + TMP;
    %end
    %SINALAMOSTRADO = SINALAMOSTRADO + SINAL + fliplr(SINALAMOSTRADO);
        
    SINALQUANTIZADO = fftshift(fft(sinalquantizado, length(f2)))/fa; 
    
    % plot do sinal original na frequ�ncia 
    if handles.metricdata.checkbox_sinalOriginal
        plot(handles.axes_freq,f,abs(SINAL));
        set(handles.axes_freq,'NextPlot','add');
    end

    % plot do sinal amostrado na frequ�ncia 
    if handles.metricdata.checkbox_sinalAmostrado
        plot(handles.axes_freq,f,abs(SINALAMOSTRADO), 'r');
        set(handles.axes_freq,'NextPlot','add');    
    end
    
    % plot do sinal quantizado na frequ�ncia 
    if handles.metricdata.checkbox_sinalQuantizado
       plot(handles.axes_freq,f2,abs(SINALQUANTIZADO),'k');
    end
    
    set(handles.axes_freq,'NextPlot','replace'); % equivalente ao hold off
    
    % define os limites do eixo X no tempo para ondas predefinidas ou arquivo de som
    if tiposinal ~= 6
        set(handles.axes_freq,'xlim', [-4.5*fa 4.5*fa]);
    else
        set(handles.axes_freq,'xlim', [-40000 40000]);
    end

    % plot da curva entrada x sa�da
    amplitudex=[4*amplitudes(1) 2*amplitudes(1) amplitudes 2*amplitudes(nNiveis) 4*amplitudes(nNiveis)];
    amplitudey=[amplitudes(1) amplitudes(1) amplitudes amplitudes(nNiveis) amplitudes(nNiveis)];
    stairs(handles.axes_curva,amplitudex-(mp/nNiveis),amplitudey);       
    set(handles.axes_curva,'Ylim',[-mp mp])
    set(handles.axes_curva,'Xlim',[-2*mp 2*mp])
    grid on;
    axis equal;
    clc



end